--
-- Licensed under the OPENdj Software License.  See LICENSE file for details.
-- Copyright 2001 DAX Interactive, LLC.
--

CREATE TABLE Mood (
    mid          SERIAL PRIMARY KEY,
    mname        VARCHAR(50)
);

CREATE TABLE Genre (
    gid SERIAL PRIMARY KEY,
    gname VARCHAR(50) UNIQUE NOT NULL
);

CREATE TABLE Channel (
    cid           SERIAL PRIMARY KEY,
    name          VARCHAR(50) UNIQUE NOT NULL,
    path          VARCHAR(50) UNIQUE NOT NULL,
    isreal        SMALLINT NOT NULL DEFAULT 1,
    description   TEXT NOT NULL,
    djdescription TEXT NOT NULL
);
CREATE INDEX channel_idx_name ON Channel(name);
CREATE INDEX channel_idx_path ON Channel(path);

CREATE TABLE Dj (
    djid            SERIAL PRIMARY KEY,
    genre           INTEGER NOT NULL,
    joindate        BIGINT DEFAULT NULL,
    firstbroadcast  BIGINT DEFAULT NULL,
    numbroadcasts   INTEGER NOT NULL DEFAULT 0,
    totallisteners  INTEGER NOT NULL DEFAULT 0,
    numsubs         INTEGER NOT NULL DEFAULT 0,
    numcomments     INTEGER NOT NULL DEFAULT 0,
    numshows        INTEGER NOT NULL DEFAULT 0,
    numreruns       INTEGER NOT NULL DEFAULT 0,
    numstartlate    INTEGER NOT NULL DEFAULT 0,
    numendearly     INTEGER NOT NULL DEFAULT 0,
    numnoshow       INTEGER NOT NULL DEFAULT 0,
    maxweeklyblocks INTEGER NOT NULL DEFAULT 32,
    maxsingleblocks INTEGER NOT NULL DEFAULT 64,
    karma           BIGINT NOT NULL DEFAULT 0,
    ipaddr          VARCHAR(20) NOT NULL,
    email           VARCHAR(100) UNIQUE NOT NULL,
    password        VARCHAR(100),
    showname        VARCHAR(100) UNIQUE,
    djname          VARCHAR(100) UNIQUE,
    city            VARCHAR(100),
    state           VARCHAR(20),
    country         VARCHAR(80),
    phone           VARCHAR(20),
    musicdesc       TEXT,
    webpage         TEXT,
    CONSTRAINT fk_dj_genre FOREIGN KEY (genre) REFERENCES Genre(gid)
);
CREATE INDEX dj_idx_ipaddr         ON Dj(ipaddr);
CREATE INDEX dj_idx_genre          ON Dj(genre);
CREATE INDEX dj_idx_numshows       ON Dj(numshows);
CREATE INDEX dj_idx_numreruns      ON Dj(numreruns);
CREATE INDEX dj_idx_karma          ON Dj(karma);
CREATE INDEX dj_idx_totallisteners ON Dj(totallisteners);

CREATE TABLE ScheduleBlock (
    bid         SERIAL PRIMARY KEY,
    channelid   INTEGER NOT NULL,
    day	        SMALLINT NOT NULL,
    attributes	INTEGER NOT NULL DEFAULT 0,
    starttime	INTEGER NOT NULL DEFAULT 0,
    CONSTRAINT fk_scheduleblock_channelid FOREIGN KEY (channelid) REFERENCES Channel(cid)
);
CREATE INDEX scheduleblock_idx_channelid  ON ScheduleBlock(channelid);
CREATE INDEX scheduleblock_idx_day        ON ScheduleBlock(day);
CREATE INDEX scheduleblock_idx_attributes ON ScheduleBlock(attributes);
CREATE INDEX scheduleblock_idx_starttime  ON ScheduleBlock(starttime);

CREATE TABLE Scheduled (
    sid         SERIAL PRIMARY KEY,
    bid         INTEGER NOT NULL,
    djid        INTEGER NOT NULL DEFAULT 1,
    issingle    SMALLINT DEFAULT 0,
    singledate  DATE NOT NULL,
    startlease  DATE,
    endlease    DATE,
    isactive    SMALLINT NOT NULL,
    ispublished SMALLINT NOT NULL,
    CONSTRAINT fk_scheduled_bid FOREIGN KEY  (bid)  REFERENCES ScheduleBlock(bid),
    CONSTRAINT fk_scheduled_djid FOREIGN KEY (djid) REFERENCES Dj(djid)
);
CREATE INDEX scheduled_idx_bid         ON Scheduled(bid);
CREATE INDEX scheduled_idx_djid        ON Scheduled(djid);
CREATE INDEX scheduled_idx_issingle    ON Scheduled(issingle);
CREATE INDEX scheduled_idx_singledate  ON Scheduled(singledate);
CREATE INDEX scheduled_idx_startlease  ON Scheduled(startlease);
CREATE INDEX scheduled_idx_endlease    ON Scheduled(endlease);
CREATE INDEX scheduled_idx_isactive    ON Scheduled(isactive);
CREATE INDEX scheduled_idx_ispublished ON Scheduled(ispublished);
CREATE UNIQUE INDEX scheduled_idx_uniquesingle ON Scheduled(bid, singledate);
CREATE UNIQUE INDEX scheduled_idx_uniqueweekly ON Scheduled(bid, startlease);

CREATE TABLE ArchivedShow (
    archid        SERIAL PRIMARY KEY,
    djid          INTEGER NOT NULL,
    genre         INTEGER NOT NULL,
    djmood        INTEGER NOT NULL DEFAULT 1,
    starttime     BIGINT NOT NULL,
    actualstart   BIGINT,
    endtime       BIGINT NOT NULL,
    lastaired     BIGINT NOT NULL,
    channelid     INTEGER NOT NULL, 
    numlisteners  INTEGER NOT NULL DEFAULT 0,
    numcomments   INTEGER NOT NULL DEFAULT 0,
    bitrate       INTEGER NOT NULL DEFAULT 0,
    rconntotal    INTEGER NOT NULL DEFAULT 0,
    rconncount    INTEGER NOT NULL DEFAULT 0,
    rconnavg      INTEGER NOT NULL DEFAULT 0,
    raudiototal   INTEGER NOT NULL DEFAULT 0,
    raudiocount   INTEGER NOT NULL DEFAULT 0,
    raudioavg     INTEGER NOT NULL DEFAULT 0,
    rstyletotal   INTEGER NOT NULL DEFAULT 0,
    rstylecount   INTEGER NOT NULL DEFAULT 0,
    rstyleavg     INTEGER NOT NULL DEFAULT 0,
    iscompleted   SMALLINT NOT NULL DEFAULT 0,
    iskeeper      SMALLINT NOT NULL DEFAULT 0,
    isrerunnable  SMALLINT NOT NULL DEFAULT 0,
    karmaearned   INTEGER NOT NULL DEFAULT 0,
    moodval       VARCHAR(50),
    genreval      VARCHAR(50),
    djname        VARCHAR(100),
    showname      VARCHAR(100),
    showfile      VARCHAR(255),
    showurl       VARCHAR(255),
    transcripturl VARCHAR(255),
    djcomments    TEXT,
    CONSTRAINT fk_archivedshow_djid      FOREIGN KEY (djid)      REFERENCES Dj(djid),
    CONSTRAINT fk_archivedshow_genre     FOREIGN KEY (genre)     REFERENCES Genre(gid),
    CONSTRAINT fk_archivedshow_djmood    FOREIGN KEY (djmood)    REFERENCES Mood(mid),
    CONSTRAINT fk_archivedshow_channelid FOREIGN KEY (channelid) REFERENCES Channel(cid)
);
CREATE INDEX archivedshow_idx_djid ON ArchivedShow(djid);
CREATE INDEX archivedshow_idx_genre ON ArchivedShow(genre);
CREATE INDEX archivedshow_idx_djmood ON ArchivedShow(djmood);
CREATE INDEX archivedshow_idx_channelid ON ArchivedShow(channelid);
CREATE INDEX archivedshow_idx_starttime ON ArchivedShow(starttime);
CREATE INDEX archivedshow_idx_endtime ON ArchivedShow(endtime);
CREATE INDEX archivedshow_idx_lastaired ON ArchivedShow(lastaired);
CREATE INDEX archivedshow_idx_numlisteners ON ArchivedShow(numlisteners);
CREATE INDEX archivedshow_idx_numcomments ON ArchivedShow(numcomments);
CREATE INDEX archivedshow_idx_rconncount ON ArchivedShow(rconncount);
CREATE INDEX archivedshow_idx_rconnavg ON ArchivedShow(rconnavg);
CREATE INDEX archivedshow_idx_raudiocount ON ArchivedShow(raudiocount);
CREATE INDEX archivedshow_idx_raudioavg ON ArchivedShow(raudioavg);
CREATE INDEX archivedshow_idx_rstylecount ON ArchivedShow(rstylecount);
CREATE INDEX archivedshow_idx_rstyleavg ON ArchivedShow(rstyleavg);
CREATE INDEX archivedshow_idx_iscompleted ON ArchivedShow(iscompleted);
CREATE INDEX archivedshow_idx_iskeeper ON ArchivedShow(iskeeper);
CREATE INDEX archivedshow_idx_isrerunnable ON ArchivedShow(isrerunnable);
CREATE INDEX archivedshow_idx_karmaearned ON ArchivedShow(karmaearned);

CREATE TABLE PublishedSchedule (
    psid        SERIAL PRIMARY KEY,
    channelid   INTEGER NOT NULL,
    djid        INTEGER NOT NULL,
    edjid       INTEGER NOT NULL,
    genre       INTEGER NOT NULL,
    starttime	BIGINT NOT NULL,
    endtime     BIGINT NOT NULL,
    status      SMALLINT NOT NULL DEFAULT 0,
    numweeks    INTEGER NOT NULL DEFAULT 0,
    archid      INTEGER NOT NULL DEFAULT 1,
    channelname VARCHAR(50) NOT NULL,
    channelpath VARCHAR(50) NOT NULL,
    ipaddr      VARCHAR(20) NOT NULL,
    djemail     VARCHAR(100) NOT NULL,
    djname      VARCHAR(100) NOT NULL,
    showname    VARCHAR(100) NOT NULL,
    genreval    VARCHAR(50) NOT NULL,
    CONSTRAINT fk_publishedschedule_channelid FOREIGN KEY (channelid) REFERENCES Channel(cid),
    CONSTRAINT fk_publishedschedule_djid      FOREIGN KEY (djid)      REFERENCES Dj(djid),
    CONSTRAINT fk_publishedschedule_edjid     FOREIGN KEY (edjid)     REFERENCES Dj(djid),
    CONSTRAINT fk_publishedschedule_genre     FOREIGN KEY (genre)     REFERENCES Genre(gid),
    CONSTRAINT fk_publishedschedule_archid    FOREIGN KEY (archid)    REFERENCES ArchivedShow(archid)
);
CREATE INDEX publishedschedule_idx_channelid ON PublishedSchedule(channelid);
CREATE INDEX publishedschedule_idx_djid      ON PublishedSchedule(djid);
CREATE INDEX publishedschedule_idx_edjid     ON PublishedSchedule(edjid);
CREATE INDEX publishedschedule_idx_starttime ON PublishedSchedule(starttime);
CREATE INDEX publishedschedule_idx_endtime   ON PublishedSchedule(endtime);
CREATE INDEX publishedschedule_idx_status    ON PublishedSchedule(status);
CREATE INDEX publishedschedule_idx_archid    ON PublishedSchedule(archid);

CREATE TABLE OnAirStatus (
    psid           INTEGER UNIQUE NOT NULL,
    bitrate        INTEGER NOT NULL DEFAULT 0,
    djid           INTEGER NOT NULL,
    edjid          INTEGER NOT NULL,
    starttime      BIGINT NOT NULL,
    endtime        BIGINT NOT NULL,
    numweeks       INTEGER NOT NULL DEFAULT 0,
    status         SMALLINT NOT NULL DEFAULT 0,
    onairtime      BIGINT NOT NULL DEFAULT 0,
    lastonairtime  BIGINT NOT NULL DEFAULT 0,
    lastsampletime BIGINT NOT NULL DEFAULT 0,
    samplecount    INTEGER NOT NULL DEFAULT 0,
    onaircount     INTEGER NOT NULL DEFAULT 0,
    listenercount  INTEGER NOT NULL DEFAULT 0,
    recentlcount   INTEGER NOT NULL DEFAULT 0,
    channelpath    VARCHAR(50) PRIMARY KEY,
    CONSTRAINT fk_onairstatus_psid        FOREIGN KEY (psid)        REFERENCES PublishedSchedule(psid),
    CONSTRAINT fk_onairstatus_djid        FOREIGN KEY (djid)        REFERENCES Dj(djid),
    CONSTRAINT fk_onairstatus_edjid       FOREIGN KEY (edjid)       REFERENCES Dj(djid),
    CONSTRAINT fk_onairstatus_channelpath FOREIGN KEY (channelpath) REFERENCES Channel(path)
);
CREATE INDEX onairstatus_idx_psid  ON OnAirStatus(psid);
CREATE INDEX onairstatus_idx_djid  ON OnAirStatus(djid);
CREATE INDEX onairstatus_idx_edjid ON OnAirStatus(edjid);

CREATE TABLE ChatRoom (
    roomname      VARCHAR(50) PRIMARY KEY,
    maxchatters   INTEGER NOT NULL DEFAULT 100,
    ispublic      SMALLINT DEFAULT 0,
    moderatordjid INTEGER  DEFAULT NULL,
    description   VARCHAR(200) DEFAULT NULL
);

CREATE TABLE ChatComments (
    commentid SERIAL PRIMARY KEY,
    archid    INTEGER NOT NULL DEFAULT 0,
    posted    BIGINT NOT NULL,
    roomname  VARCHAR(50) NOT NULL,
    fromname  VARCHAR(100),
    message   TEXT,
    CONSTRAINT fk_chatcomments_archid   FOREIGN KEY (archid)   REFERENCES ArchivedShow(archid),
    CONSTRAINT fk_chatcomments_roomname FOREIGN KEY (roomname) REFERENCES ChatRoom(roomname)
);
CREATE INDEX chatcomments_idx_roomname ON ChatComments(roomname);
CREATE INDEX chatcomments_idx_archid   ON ChatComments(archid);
CREATE INDEX chatcomments_idx_posted   ON ChatComments(posted);

CREATE TABLE ChatBoot (
    cbid       SERIAL PRIMARY KEY,
    djid       INTEGER NOT NULL,
    boottime   BIGINT NOT NULL,
    ipaddr     VARCHAR(20) NOT NULL,
    name       VARCHAR(100) NOT NULL
);
CREATE INDEX chatboot_idx_djid     ON ChatBoot(djid);
CREATE INDEX chatboot_idx_boottime ON ChatBoot(boottime);
CREATE INDEX chatboot_idx_ipaddr   ON ChatBoot(ipaddr);
CREATE INDEX chatboot_idx_name     ON ChatBoot(name);

CREATE TABLE AsyncMail (
    amid       SERIAL PRIMARY KEY,
    status     SMALLINT NOT NULL,
    queuetime  BIGINT NOT NULL,
    errorcount SMALLINT NOT NULL DEFAULT 0,
    nextsend   BIGINT NOT NULL,
    client     VARCHAR(20) NOT NULL,
    fromemail  VARCHAR(250) NOT NULL,
    subject    VARCHAR(250) NOT NULL,
    toemails   VARCHAR(255) NOT NULL,
    message    TEXT
);
CREATE INDEX asyncmail_idx_status     ON AsyncMail(status);
CREATE INDEX asyncmail_idx_client     ON AsyncMail(client);
CREATE INDEX asyncmail_idx_queuetime  ON AsyncMail(queuetime);
CREATE INDEX asyncmail_idx_errorcount ON AsyncMail(errorcount);
CREATE INDEX asyncmail_idx_nextsend   ON AsyncMail(nextsend);

CREATE TABLE PowerStatus (
    batterycharge DECIMAL(5,2),
    timeleft     DECIMAL(5,2)
);
